from rest_framework import serializers

from virtualization.api.serializers import NestedClusterSerializer, NestedVirtualMachineSerializer
from dcim.api.serializers import NestedDeviceSerializer
from netbox.api.serializers import NetBoxModelSerializer, WritableNestedSerializer
from netbox.api.fields import SerializedPKRelatedField
from ..models import StoragePool, LUN, StorageSession, Datastore, VMDK


#
# Nested serializers
#

class NestedStoragePoolSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_storage-api:storagepool-detail'
    )

    class Meta:
        model = StoragePool
        fields = ('id', 'url', 'display', 'name')


class NestedLUNSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_storage-api:lun-detail'
    )

    class Meta:
        model = LUN
        fields = ('id', 'url', 'display', 'name')


class NestedDatastoreSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_storage-api:datastore-detail'
    )

    class Meta:
        model = Datastore
        fields = ('id', 'url', 'display', 'name')


class NestedStorageSessionSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_storage-api:storagesession-detail'
    )

    class Meta:
        model = StorageSession
        fields = ('id', 'url', 'display', 'name')


class NestedVMDKSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_storage-api:vmdk-detail'
    )

    class Meta:
        model = VMDK
        fields = ('id', 'url', 'display', 'name')


#
# Regular serializers
#

class StoragePoolSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_storage-api:storagepool-detail'
    )
    device = NestedDeviceSerializer()

    class Meta:
        model = StoragePool
        fields = (
            'id', 'url', 'display', 'name', 'size', 'device', 'description',
            'tags', 'custom_fields', 'created', 'last_updated',
        )


class LUNSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_storage-api:lun-detail'
    )
    storage_pool = NestedStoragePoolSerializer()

    class Meta:
        model = LUN
        fields = (
            'id', 'url', 'display', 'name', 'size', 'storage_pool', 'wwn',
            'description', 'tags', 'custom_fields',
            'created', 'last_updated',
        )


class DatastoreSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_storage-api:datastore-detail'
    )
    lun = SerializedPKRelatedField(
        queryset=LUN.objects.all(),
        serializer=NestedLUNSerializer,
        many=True
    )

    class Meta:
        model = Datastore
        fields = (
            'id', 'url', 'display', 'name', 'lun',
            'description', 'tags', 'custom_fields', 'created', 'last_updated',
        )


class StorageSessionSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_storage-api:storagesession-detail'
    )
    cluster = NestedClusterSerializer()
    datastores = SerializedPKRelatedField(
        queryset=Datastore.objects.all(),
        serializer=NestedDatastoreSerializer,
        many=True
    )

    class Meta:
        model = StorageSession
        fields = (
            'id', 'url', 'display', 'name', 'cluster', 'datastores',
            'description', 'tags', 'custom_fields', 'created', 'last_updated',
        )


class VMDKSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_storage-api:vmdk-detail'
    )
    datastore = NestedDatastoreSerializer()
    vm = NestedVirtualMachineSerializer()

    class Meta:
        model = VMDK
        fields = (
            'id', 'url', 'display', 'vm', 'name', 'datastore',
            'size', 'tags', 'custom_fields', 'created', 'last_updated',
        )



from netbox.api.routers import NetBoxRouter
from . import views


app_name = 'netbox_storage'

router = NetBoxRouter()
router.register('storagepool', views.StoragePoolViewSet)
router.register('lun', views.LUNViewSet)
router.register('datastore', views.DatastoreViewSet)
router.register('storagesession', views.StorageSessionViewSet)
router.register('vmdk', views.VMDKViewSet)

urlpatterns = router.urls



from netbox.filtersets import NetBoxModelFilterSet
import django_filters
from virtualization.models import VirtualMachine
from .models import StoragePool, LUN, StorageSession, Datastore, VMDK


class StoragePoolFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = StoragePool
        fields = ('id', 'name', 'device')

    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)


class LUNFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = LUN
        fields = ('id', 'storage_pool', 'name', 'wwn',)

    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)


class DatastoreFilterSet(NetBoxModelFilterSet):
    reachable_by_vm = django_filters.ModelMultipleChoiceFilter(
        field_name='storage_sessions__cluster__virtual_machines',
        queryset=VirtualMachine.objects.all(),
        label='Reachable by these Virtual Machines'
    )

    class Meta:
        model = Datastore
        fields = ('id', 'lun', 'name', 'reachable_by_vm',)

    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)


class StorageSessionFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = StorageSession
        fields = (
            'id', 'name', 'cluster', 'datastores',
        )

    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)


class VMDKFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = VMDK
        fields = (
            'id', 'name', 'vm', 'datastore',
        )

    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)



from extras.plugins import PluginConfig


class NetBoxStorageConfig(PluginConfig):
    name = 'netbox_storage'
    verbose_name = ' NetBox Storage'
    description = 'Netbox Storage Administration Plugin'
    version = '0.6.2'
    base_url = 'storage'
    min_version = "3.4.0"
    author = 'Gabor Somogyvari'


config = NetBoxStorageConfig



from netbox.views import generic
from . import filtersets, forms, models, tables


#
# StoragePool views
#

class StoragePoolView(generic.ObjectView):
    queryset = models.StoragePool.objects.all()

    def get_extra_context(self, request, instance):
        table = tables.LUNTable(instance.luns.all())
        table.configure(request)

        return {
            'luns_table': table,
        }


class StoragePoolListView(generic.ObjectListView):
    queryset = models.StoragePool.objects.all()
    table = tables.StoragePoolTable
    filterset = filtersets.StoragePoolFilterSet
    filterset_form = forms.StoragePoolFilterForm


class StoragePoolEditView(generic.ObjectEditView):
    queryset = models.StoragePool.objects.all()
    form = forms.StoragePoolForm


class StoragePoolDeleteView(generic.ObjectDeleteView):
    queryset = models.StoragePool.objects.all()


class StoragePoolBulkDeleteView(generic.BulkDeleteView):
    queryset = models.StoragePool.objects.all()
    table = tables.StoragePoolTable
    filterset = filtersets.StoragePoolFilterSet


class StoragePoolImportView(generic.BulkImportView):
    queryset = models.StoragePool.objects.all()
    model_form = forms.StoragePoolCSVForm
    table = tables.StoragePoolTable


#
# LUN views
#

class LUNView(generic.ObjectView):
    queryset = models.LUN.objects.all()

    def get_extra_context(self, request, instance):
        datastores_table = tables.DatastoreTable(instance.datastores.all())
        datastores_table.configure(request)

        return {
            'datastores_table': datastores_table,
        }


class LUNListView(generic.ObjectListView):
    queryset = models.LUN.objects.all()
    table = tables.LUNTable
    filterset = filtersets.LUNFilterSet
    filterset_form = forms.LUNFilterForm


class LUNEditView(generic.ObjectEditView):
    queryset = models.LUN.objects.all()
    form = forms.LUNForm


class LUNDeleteView(generic.ObjectDeleteView):
    queryset = models.LUN.objects.all()


class LUNBulkDeleteView(generic.BulkDeleteView):
    queryset = models.LUN.objects.all()
    table = tables.LUNTable
    filterset = filtersets.LUNFilterSet


class LUNImportView(generic.BulkImportView):
    queryset = models.LUN.objects.all()
    model_form = forms.LUNCSVForm
    table = tables.LUNTable


#
# StorageLUNGroup views
#

class DatastoreView(generic.ObjectView):
    queryset = models.Datastore.objects.all()

    def get_extra_context(self, request, instance):
        luns_table = tables.LUNTable(instance.lun.all())
        luns_table.configure(request)

        sessions_table = tables.StorageSessionTable(instance.storage_sessions.all())
        sessions_table.configure(request)

        vmdks_table = tables.VMDKTable(instance.vmdks.all())
        vmdks_table.configure(request)

        return {
            'luns_table': luns_table,
            'sessions_table': sessions_table,
            'vmdks_table': vmdks_table,
        }


class DatastoreListView(generic.ObjectListView):
    queryset = models.Datastore.objects.all()
    table = tables.DatastoreTable
    filterset = filtersets.DatastoreFilterSet
    filterset_form = forms.DatastoreFilterForm


class DatastoreEditView(generic.ObjectEditView):
    queryset = models.Datastore.objects.all()
    form = forms.DatastoreForm


class DatastoreDeleteView(generic.ObjectDeleteView):
    queryset = models.Datastore.objects.all()


class DatastoreBulkDeleteView(generic.BulkDeleteView):
    queryset = models.Datastore.objects.all()
    table = tables.DatastoreTable
    filterset = filtersets.DatastoreFilterSet


class DatastoreImportView(generic.BulkImportView):
    queryset = models.Datastore.objects.all()
    model_form = forms.DatastoreCSVForm
    table = tables.DatastoreTable


#
# StorageSession views
#

class StorageSessionView(generic.ObjectView):
    queryset = models.StorageSession.objects.all()


class StorageSessionListView(generic.ObjectListView):
    queryset = models.StorageSession.objects.all()
    table = tables.StorageSessionTable
    filterset = filtersets.StorageSessionFilterSet
    filterset_form = forms.StorageSessionFilterForm


class StorageSessionEditView(generic.ObjectEditView):
    queryset = models.StorageSession.objects.all()
    form = forms.StorageSessionForm


class StorageSessionDeleteView(generic.ObjectDeleteView):
    queryset = models.StorageSession.objects.all()


class StorageSessionBulkDeleteView(generic.BulkDeleteView):
    queryset = models.StorageSession.objects.all()
    table = tables.StorageSessionTable
    filterset = filtersets.StorageSessionFilterSet


class StorageSessionImportView(generic.BulkImportView):
    queryset = models.StorageSession.objects.all()
    model_form = forms.StorageSessionCSVForm
    table = tables.StorageSessionTable


#
# VMDK views
#

class VMDKView(generic.ObjectView):
    queryset = models.VMDK.objects.all()


class VMDKListView(generic.ObjectListView):
    queryset = models.VMDK.objects.all()
    table = tables.VMDKTable
    filterset = filtersets.VMDKFilterSet
    filterset_form = forms.VMDKFilterForm


class VMDKEditView(generic.ObjectEditView):
    queryset = models.VMDK.objects.all()
    form = forms.VMDKForm


class VMDKDeleteView(generic.ObjectDeleteView):
    queryset = models.VMDK.objects.all()


class VMDKBulkDeleteView(generic.BulkDeleteView):
    queryset = models.VMDK.objects.all()
    filterset = filtersets.VMDKFilterSet
    table = tables.VMDKTable


class VMDKImportView(generic.BulkImportView):
    queryset = models.VMDK.objects.all()
    model_form = forms.VMDKCSVForm
    table = tables.VMDKTable



from extras.plugins import PluginTemplateExtension
from .models import VMDK


class VMVMDKCard(PluginTemplateExtension):
    model = 'virtualization.virtualmachine'

    def left_page(self):
        return self.render('netbox_storage/vm_vmdk_extend.html',
                           extra_context={'object': self.context['object']})


template_extensions = [VMVMDKCard]



from django.urls import path
from . import models, views
from netbox.views.generic import ObjectChangeLogView


urlpatterns = (

    # Storage pools
    path('storagepool/', views.StoragePoolListView.as_view(), name='storagepool_list'),
    path('storagepool/add/', views.StoragePoolEditView.as_view(), name='storagepool_add'),
    path('storagepool/import/', views.StoragePoolImportView.as_view(), name='storagepool_import'),
    path('storagepool/<int:pk>/', views.StoragePoolView.as_view(), name='storagepool'),
    path('storagepool/<int:pk>/edit/', views.StoragePoolEditView.as_view(), name='storagepool_edit'),
    path('storagepool/<int:pk>/delete/', views.StoragePoolDeleteView.as_view(), name='storagepool_delete'),
    path('storagepool/delete/', views.StoragePoolBulkDeleteView.as_view(), name='storagepool_bulk_delete'),
    path('storagepool/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='storagepool_changelog', kwargs={
        'model': models.StoragePool
    }),

    # LUNs
    path('lun/', views.LUNListView.as_view(), name='lun_list'),
    path('lun/add/', views.LUNEditView.as_view(), name='lun_add'),
    path('lun/import/', views.LUNImportView.as_view(), name='lun_import'),
    path('lun/<int:pk>/', views.LUNView.as_view(), name='lun'),
    path('lun/<int:pk>/edit/', views.LUNEditView.as_view(), name='lun_edit'),
    path('lun/<int:pk>/delete/', views.LUNDeleteView.as_view(), name='lun_delete'),
    path('lun/delete/', views.LUNBulkDeleteView.as_view(), name='lun_bulk_delete'),
    path('lun/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='lun_changelog', kwargs={
        'model': models.LUN
    }),

    # Datastores
    path('datastore/', views.DatastoreListView.as_view(), name='datastore_list'),
    path('datastore/add/', views.DatastoreEditView.as_view(), name='datastore_add'),
    path('datastore/import/', views.DatastoreImportView.as_view(), name='datastore_import'),
    path('datastore/<int:pk>/', views.DatastoreView.as_view(), name='datastore'),
    path('datastore/<int:pk>/edit/', views.DatastoreEditView.as_view(), name='datastore_edit'),
    path('datastore/<int:pk>/delete/', views.DatastoreDeleteView.as_view(), name='datastore_delete'),
    path('datastore/delete/', views.DatastoreBulkDeleteView.as_view(), name='datastore_bulk_delete'),
    path('datastore/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='datastore_changelog', kwargs={
        'model': models.Datastore
    }),

    # Storage sessions
    path('storagesession/', views.StorageSessionListView.as_view(), name='storagesession_list'),
    path('storagesession/add/', views.StorageSessionEditView.as_view(), name='storagesession_add'),
    path('storagesession/import/', views.StorageSessionImportView.as_view(), name='storagesession_import'),
    path('storagesession/<int:pk>/', views.StorageSessionView.as_view(), name='storagesession'),
    path('storagesession/<int:pk>/edit/', views.StorageSessionEditView.as_view(), name='storagesession_edit'),
    path('storagesession/<int:pk>/delete/', views.StorageSessionDeleteView.as_view(), name='storagesession_delete'),
    path('storagesession/delete/', views.StorageSessionBulkDeleteView.as_view(), name='storagesession_bulk_delete'),
    path('storagesession/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='storagesession_changelog', kwargs={
        'model': models.StorageSession
    }),

    # VMDK
    path('vmdk/', views.VMDKListView.as_view(), name='vmdk_list'),
    path('vmdk/add/', views.VMDKEditView.as_view(), name='vmdk_add'),
    path('vmdk/import/', views.VMDKImportView.as_view(), name='vmdk_import'),
    path('vmdk/<int:pk>/', views.VMDKView.as_view(), name='vmdk'),
    path('vmdk/<int:pk>/edit/', views.VMDKEditView.as_view(), name='vmdk_edit'),
    path('vmdk/<int:pk>/delete/', views.VMDKDeleteView.as_view(), name='vmdk_delete'),
    path('vmdk/delete/', views.VMDKBulkDeleteView.as_view(), name='vmdk_bulk_delete'),
    path('vmdk/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='vmdk_changelog', kwargs={
        'model': models.VMDK
    }),
)



from django import forms
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm, NetBoxModelCSVForm
from utilities.forms.fields import DynamicModelChoiceField, CSVModelChoiceField, DynamicModelMultipleChoiceField, CSVModelMultipleChoiceField
from dcim.models import Device
from virtualization.models import Cluster, VirtualMachine
from .models import StoragePool, StorageSession, LUN, Datastore, VMDK


#
# Regular forms
#

class StoragePoolForm(NetBoxModelForm):
    device = DynamicModelChoiceField(
        queryset=Device.objects.all()
    )

    class Meta:
        model = StoragePool
        fields = ('name', 'size', 'device', 'description', 'tags')


class LUNForm(NetBoxModelForm):
    storage_pool = DynamicModelChoiceField(
        queryset=StoragePool.objects.all()
    )

    class Meta:
        model = LUN
        fields = ('storage_pool', 'name', 'size', 'wwn', 'description')


class DatastoreForm(NetBoxModelForm):
    lun = DynamicModelMultipleChoiceField(
        queryset=LUN.objects.all()
    )

    class Meta:
        model = LUN
        fields = ('lun', 'name', 'description')


class StorageSessionForm(NetBoxModelForm):
    cluster = DynamicModelChoiceField(
        queryset=Cluster.objects.all(),
    )
    datastores = DynamicModelMultipleChoiceField(
        queryset=Datastore.objects.all()
    )

    class Meta:
        model = StorageSession
        fields = (
            'name', 'cluster', 'datastores', 'description'
        )


class VMDKForm(NetBoxModelForm):
    vm = DynamicModelChoiceField(
        queryset=VirtualMachine.objects.all(),
        label='Virtual Machine'
    )
    datastore = DynamicModelChoiceField(
        queryset=Datastore.objects.all(),
        query_params={
            'reachable_by_vm': '$vm'
        }
    )

    class Meta:
        model = VMDK
        fields = (
            'vm', 'datastore', 'name', 'size',
        )


#
# Filter forms
#

class StoragePoolFilterForm(NetBoxModelFilterSetForm):
    model = StoragePool
    device = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False
    )
    name = forms.CharField(
        required=False
    )


class LUNFilterForm(NetBoxModelFilterSetForm):
    model = LUN
    storage_pool = DynamicModelMultipleChoiceField(
        queryset=StoragePool.objects.all(),
        required=False
    )
    name = forms.CharField(
        required=False
    )
    wwn = forms.CharField(
        required=False,
        label='WWN'
    )


class DatastoreFilterForm(NetBoxModelFilterSetForm):
    model = Datastore
    lun = DynamicModelMultipleChoiceField(
        queryset=LUN.objects.all(),
        required=False
    )
    name = forms.CharField(
        required=False
    )
    reachable_by_vm = DynamicModelMultipleChoiceField(
        queryset=VirtualMachine.objects.all(),
        required=False,
        label='Reachable by these Virtual Machines'
    )


class StorageSessionFilterForm(NetBoxModelFilterSetForm):
    model = StorageSession
    cluster = DynamicModelMultipleChoiceField(
        queryset=Cluster.objects.all(),
        required=False
    )
    datastores = DynamicModelMultipleChoiceField(
        queryset=Datastore.objects.all(),
        required=False
    )
    name = forms.CharField(
        required=False
    )


class VMDKFilterForm(NetBoxModelFilterSetForm):
    model = VMDK
    datastore = DynamicModelMultipleChoiceField(
        queryset=Datastore.objects.all(),
        required=False
    )
    vm = DynamicModelMultipleChoiceField(
        queryset=VirtualMachine.objects.all(),
        required=False
    )
    name = forms.CharField(
        required=False
    )


#
# CSV Forms
#

class StoragePoolCSVForm(NetBoxModelCSVForm):
    device = CSVModelChoiceField(
        queryset=Device.objects.all(),
        to_field_name='name',
    )

    class Meta:
        model = StoragePool
        fields = ('name', 'size', 'device', 'description')


class LUNCSVForm(NetBoxModelCSVForm):
    storage_pool = CSVModelChoiceField(
        queryset=StoragePool.objects.all(),
        to_field_name='name',
    )

    class Meta:
        model = LUN
        fields = ('storage_pool', 'name', 'size', 'wwn', 'description')


class DatastoreCSVForm(NetBoxModelCSVForm):
    lun = CSVModelMultipleChoiceField(
        queryset=LUN.objects.all(),
        to_field_name='name',
        help_text='A single LUN name or multiple LUN names separated by commas ("LUN" or "LUN1,LUN2,LUN3")'
    )

    class Meta:
        model = Datastore
        fields = ('lun', 'name', 'description')


class StorageSessionCSVForm(NetBoxModelCSVForm):
    cluster = CSVModelChoiceField(
        queryset=Cluster.objects.all(),
        to_field_name='name',
    )
    datastores = CSVModelMultipleChoiceField(
        queryset=Datastore.objects.all(),
        to_field_name='name',
        help_text='A single Datastore name or multiple Datastore names separated by commas ("datastore1" or "datastore1,datastore2,datastore3")'
    )

    class Meta:
        model = StorageSession
        fields = ('cluster', 'datastores', 'name', 'description')


class VMDKCSVForm(NetBoxModelCSVForm):
    vm = CSVModelChoiceField(
        queryset=VirtualMachine.objects.all(),
        to_field_name='name',
    )
    datastore = CSVModelChoiceField(
        queryset=Datastore.objects.all(),
        to_field_name='name',
    )

    class Meta:
        model = VMDK
        fields = ('vm', 'datastore', 'name', 'size')



# Generated by Django 4.1.6 on 2023-02-18 06:38

from django.db import migrations, models
import django.db.models.deletion
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('extras', '0084_staging'),
        ('virtualization', '0034_standardize_description_comments'),
        ('dcim', '0167_module_status'),
    ]

    operations = [
        migrations.CreateModel(
            name='Datastore',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(blank=True)),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='VMDK',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100)),
                ('size', models.PositiveBigIntegerField()),
                ('datastore', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='vmdks', to='netbox_storage.datastore')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('vm', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='vmdks', to='virtualization.virtualmachine')),
            ],
            options={
                'verbose_name': 'VMDK',
                'ordering': ('datastore', 'name'),
            },
        ),
        migrations.CreateModel(
            name='StorageSession',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(blank=True)),
                ('cluster', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='storage_sessions', to='virtualization.cluster')),
                ('datastores', models.ManyToManyField(related_name='storage_sessions', to='netbox_storage.datastore')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='StoragePool',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100)),
                ('size', models.PositiveBigIntegerField()),
                ('description', models.TextField(blank=True)),
                ('device', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='dcim.device')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='LUN',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100)),
                ('size', models.PositiveBigIntegerField()),
                ('description', models.TextField(blank=True)),
                ('wwn', models.CharField(blank=True, max_length=64)),
                ('storage_pool', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='luns', to='netbox_storage.storagepool')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ('name',),
                'unique_together': {('storage_pool', 'name')},
            },
        ),
        migrations.AddField(
            model_name='datastore',
            name='lun',
            field=models.ManyToManyField(related_name='datastores', to='netbox_storage.lun'),
        ),
        migrations.AddField(
            model_name='datastore',
            name='tags',
            field=taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag'),
        ),
    ]



from django.db import models
from django.urls import reverse
from django.db.models import Sum
from netbox.models import NetBoxModel


class StoragePool(NetBoxModel):
    name = models.CharField(
        max_length=100
    )
    size = models.PositiveBigIntegerField(
        help_text='Size in bytes'
    )
    device = models.ForeignKey(
        to='dcim.Device',
        on_delete=models.PROTECT
    )
    description = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_storage:storagepool', args=[self.pk])

    def get_utilization(self):
        sum_alloc_size = self.luns.all().aggregate(Sum('size'))['size__sum']
        if sum_alloc_size:
            utilization = float(sum_alloc_size) / self.size * 100
        else:
            utilization = 0

        return utilization


class LUN(NetBoxModel):
    storage_pool = models.ForeignKey(
        to=StoragePool,
        on_delete=models.PROTECT,
        related_name='luns'
    )
    name = models.CharField(
        max_length=100
    )
    size = models.PositiveBigIntegerField(
        help_text='Size in bytes'
    )
    description = models.TextField(
        blank=True
    )
    wwn = models.CharField(
        max_length=64,
        blank=True,
        verbose_name='WWN'
    )

    class Meta:
        ordering = ('name',)
        unique_together = ('storage_pool', 'name')

    def __str__(self):
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_storage:lun', args=[self.pk])


class Datastore(NetBoxModel):
    lun = models.ManyToManyField(
        to=LUN,
        related_name='datastores'
    )
    name = models.CharField(
        max_length=100
    )
    description = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_storage:datastore', args=[self.pk])

    def get_utilization(self):
        sum_lun_size = self.lun.all().aggregate(Sum('size'))['size__sum']
        sum_vmdk_size = self.vmdks.all().aggregate(Sum('size'))['size__sum']
        if sum_lun_size and sum_vmdk_size:
            utilization = float(sum_vmdk_size) / float(sum_lun_size) * 100
        else:
            utilization = 0

        return utilization


class StorageSession(NetBoxModel):
    name = models.CharField(
        max_length=100
    )
    cluster = models.ForeignKey(
        to='virtualization.cluster',
        on_delete=models.PROTECT,
        related_name='storage_sessions'
    )
    datastores = models.ManyToManyField(
        to=Datastore,
        related_name='storage_sessions'
    )
    description = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_storage:storagesession', args=[self.pk])


class VMDK(NetBoxModel):
    vm = models.ForeignKey(
        to='virtualization.virtualmachine',
        on_delete=models.PROTECT,
        related_name='vmdks',
        verbose_name='Virtual Machine'
    )
    name = models.CharField(
        max_length=100
    )
    datastore = models.ForeignKey(
        to=Datastore,
        related_name='vmdks',
        on_delete=models.PROTECT
    )
    size = models.PositiveBigIntegerField(
        help_text='Size in bytes'
    )

    class Meta:
        ordering = ('datastore', 'name',)
        verbose_name = 'VMDK'

    def __str__(self):
        return f'{self.vm}-{self.datastore}-{self.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_storage:vmdk', args=[self.pk])



from setuptools import find_packages, setup

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='netbox-storage-plugin',
    version='0.6.2',
    description='NetBox storage plugin',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    author='Gabor Somogyvari',
    url='https://github.com/viroge/netbox-storage',
    keywords=['netbox', 'netbox-plugin'],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Development Status :: 4 - Beta',
    ]
)



import django_tables2 as tables

from django.template.defaultfilters import filesizeformat
from netbox.tables import NetBoxTable, columns
from .models import StoragePool, StorageSession, Datastore, LUN, VMDK


class UtilizationColumn(columns.UtilizationColumn):
    template_code = """
    {% load helpers %}
    {% if record.pk %}
      {% utilization_graph value %}
    {% endif %}
    """


class StoragePoolTable(NetBoxTable):
    name = tables.Column(
        linkify=True
    )
    utilization = UtilizationColumn(
        accessor='get_utilization',
        orderable=False
    )
    device = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = StoragePool
        fields = (
            'pk', 'id', 'name', 'size', 'utilization', 'device', 'description',
            'actions'
        )
        default_columns = ('name', 'device', 'size', 'utilization')

    def render_size(self, value):
        return filesizeformat(value)


class LUNTable(NetBoxTable):
    name = tables.Column(
        linkify=True
    )
    storage_pool = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = LUN
        fields = (
            'pk', 'id', 'name', 'storage_pool', 'size', 'wwn', 'description', 'actions',
        )
        default_columns = (
            'name', 'storage_pool', 'size',
        )

    def render_size(self, value):
        return filesizeformat(value)


class DatastoreTable(NetBoxTable):
    name = tables.Column(
        linkify=True
    )
    lun = columns.ManyToManyColumn(
        linkify_item=True,
        verbose_name='LUNs',
    )
    utilization = UtilizationColumn(
        accessor='get_utilization',
        orderable=False
    )

    class Meta(NetBoxTable.Meta):
        model = Datastore
        fields = (
            'pk', 'id', 'name', 'lun', 'utilization', 'description', 'actions',
        )
        default_columns = (
            'name', 'lun', 'utilization',
        )


class StorageSessionTable(NetBoxTable):
    name = tables.Column(
        linkify=True
    )
    cluster = tables.Column(
        linkify=True,
    )
    datastores = columns.ManyToManyColumn(
        linkify_item=True,
        verbose_name='Datastores'
    )

    class Meta(NetBoxTable.Meta):
        model = StorageSession

        fields = (
            'pk', 'id', 'name', 'cluster', 'datastores', 'description',
        )
        default_columns = (
            'name', 'cluster', 'datastores',
        )


class VMDKTable(NetBoxTable):
    name = tables.Column(
        linkify=True
    )
    vm = tables.Column(
        linkify=True
    )
    datastore = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = VMDK

        fields = (
            'pk', 'id', 'vm', 'name', 'datastore', 'size',
        )
        default_columns = (
            'vm', 'datastore', 'name', 'size',
        )

    def render_size(self, value):
        return filesizeformat(value)



from extras.plugins import PluginMenuButton, PluginMenuItem, PluginMenu
from utilities.choices import ButtonColorChoices


storagepool_item = PluginMenuItem(
    link='plugins:netbox_storage:storagepool_list',
    link_text='Storage Pools',
    permissions=['netbox_storage.view_storagepool'],
    buttons=[
        PluginMenuButton(
            link='plugins:netbox_storage:storagepool_add',
            title='Add',
            icon_class='mdi mdi-plus-thick',
            color=ButtonColorChoices.GREEN,
            permissions=['netbox_storage.add_storagepool'],
        ),
        PluginMenuButton(
            link='plugins:netbox_storage:storagepool_import',
            title='Import',
            icon_class='mdi mdi-upload',
            color=ButtonColorChoices.CYAN,
            permissions=['netbox_storage.add_storagepool'],
        )
    ]
)

lun_item = PluginMenuItem(
    link='plugins:netbox_storage:lun_list',
    link_text='LUNs',
    permissions=['netbox_storage.view_lun'],
    buttons=[
        PluginMenuButton(
            link='plugins:netbox_storage:lun_add',
            title='Add',
            icon_class='mdi mdi-plus-thick',
            color=ButtonColorChoices.GREEN,
            permissions=['netbox_storage.add_lun'],
        ),
        PluginMenuButton(
            link='plugins:netbox_storage:lun_import',
            title='Import',
            icon_class='mdi mdi-upload',
            color=ButtonColorChoices.CYAN,
            permissions=['netbox_storage.add_lun'],
        )
    ]
)

datastore_item = PluginMenuItem(
    link='plugins:netbox_storage:datastore_list',
    link_text='Datastores',
    permissions=['netbox_storage.view_datastore'],
    buttons=[
        PluginMenuButton(
            link='plugins:netbox_storage:datastore_add',
            title='Add',
            icon_class='mdi mdi-plus-thick',
            color=ButtonColorChoices.GREEN,
            permissions=['netbox_storage.add_datastore'],
        ),
        PluginMenuButton(
            link='plugins:netbox_storage:datastore_import',
            title='Import',
            icon_class='mdi mdi-upload',
            color=ButtonColorChoices.CYAN,
            permissions=['netbox_storage.add_datastore'],
        )
    ]
)

storagesession_item = PluginMenuItem(
    link='plugins:netbox_storage:storagesession_list',
    link_text='Storage Sessions',
    permissions=['netbox_storage.view_storagesession'],
    buttons=[
        PluginMenuButton(
            link='plugins:netbox_storage:storagesession_add',
            title='Add',
            icon_class='mdi mdi-plus-thick',
            color=ButtonColorChoices.GREEN,
            permissions=['netbox_storage.add_storagesession'],
        ),
        PluginMenuButton(
            link='plugins:netbox_storage:storagesession_import',
            title='Import',
            icon_class='mdi mdi-upload',
            color=ButtonColorChoices.CYAN,
            permissions=['netbox_storage.add_storagesession'],
        )
    ]
)

vmdk_item = PluginMenuItem(
    link='plugins:netbox_storage:vmdk_list',
    link_text='VMDKs',
    permissions=['netbox_storage.view_vmdk'],
    buttons=[
        PluginMenuButton(
            link='plugins:netbox_storage:vmdk_add',
            title='Add',
            icon_class='mdi mdi-plus-thick',
            color=ButtonColorChoices.GREEN,
            permissions=['netbox_storage.add_vmdk'],
        ),
        PluginMenuButton(
            link='plugins:netbox_storage:vmdk_import',
            title='Import',
            icon_class='mdi mdi-upload',
            color=ButtonColorChoices.CYAN,
            permissions=['netbox_storage.add_vmdk'],
        )
    ]
)

menu = PluginMenu(
    label='Storage',
    groups=(
        ('Storage', (storagepool_item, lun_item)),
        ('Virtualization', (datastore_item, storagesession_item, vmdk_item))
    ),
    icon_class='mdi mdi-nas'
)



from netbox.api.viewsets import NetBoxModelViewSet

from .. import filtersets, models
from .serializers import StoragePoolSerializer, LUNSerializer, StorageSessionSerializer, DatastoreSerializer, VMDKSerializer


class StoragePoolViewSet(NetBoxModelViewSet):
    queryset = models.StoragePool.objects.prefetch_related('device', 'tags')
    serializer_class = StoragePoolSerializer
    filterset_class = filtersets.StoragePoolFilterSet


class LUNViewSet(NetBoxModelViewSet):
    queryset = models.LUN.objects.prefetch_related(
        'storage_pool', 'tags'
    )
    serializer_class = LUNSerializer
    filterset_class = filtersets.LUNFilterSet


class DatastoreViewSet(NetBoxModelViewSet):
    queryset = models.Datastore.objects.prefetch_related(
        'lun', 'tags'
    )
    serializer_class = DatastoreSerializer
    filterset_class = filtersets.DatastoreFilterSet


class StorageSessionViewSet(NetBoxModelViewSet):
    queryset = models.StorageSession.objects.prefetch_related(
        'cluster', 'datastores', 'tags'
    )
    serializer_class = StorageSessionSerializer
    filterset_class = filtersets.StorageSessionFilterSet


class VMDKViewSet(NetBoxModelViewSet):
    queryset = models.VMDK.objects.prefetch_related(
        'datastore', 'vm', 'tags'
    )
    serializer_class = VMDKSerializer
    filterset_class = filtersets.VMDKFilterSet
